package com.example.survey.controller;

import com.example.survey.dto.LoginDto;
import com.example.survey.dto.UserDto;
import com.example.survey.service.UserService;
import com.example.survey.vo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @author Pope
 */
@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    UserService userService;

    @GetMapping("/user")
    public ResultVo getUser(@RequestParam(value = "pageSize", defaultValue = "30") int pageSize,
                            @RequestParam(value = "currentPage") int currentPage,
                            @RequestParam(value = "username", required = false) String username,
                            @RequestParam(value = "phoneNumber", required = false) String phoneNumber) {
        ResultVo resultVo = new ResultVo();


        List<UserDto> userDtoList = userService.listUserLimit(username, phoneNumber, currentPage, pageSize);
        Map<String, Object> resultMap = new HashMap<>(16, 0.75F);
        resultMap.put("totalCount", userDtoList.size());
        resultMap.put("pageSize", pageSize);
        resultMap.put("currentPage", currentPage);
        resultMap.put("data", userDtoList);

        resultVo.setCode(ResultVo.SUCCESS);
        resultVo.setMsg("查询成功");
        resultVo.setData(resultMap);
        return resultVo;
    }

    @DeleteMapping("/user")
    public ResultVo deleteUser(@RequestBody DeleteUserVo deleteUserVo) {
        ResultVo resultVo = new ResultVo();

        boolean result = userService.deleteUser(deleteUserVo.getPhoneNumber());
        if (result) {
            resultVo.setCode(ResultVo.SUCCESS);
            resultVo.setMsg("删除成功");
        } else {
            resultVo.setCode(ResultVo.FAILED);
            resultVo.setMsg("删除失败");
        }
        return resultVo;
    }

    @PostMapping("/login")
    public ResultVo login(@RequestBody LoginVo loginVo) {
        ResultVo resultVo = new ResultVo();
        //用户名密码验证
        LoginDto loginDto = userService.matchAuth(loginVo);
        if (loginDto == null) {
            //登录失败
            resultVo.setCode(ResultVo.FAILED);
            resultVo.setMsg("登录失败");
            return resultVo;
        }
        resultVo.setCode(ResultVo.SUCCESS);
        resultVo.setMsg("登录成功");
        resultVo.setData(loginDto);
        return resultVo;
    }


    @PutMapping("/userRole")
    public ResultVo modifyUserRoles(@RequestBody UserRoleVo userRoleVo) {
        ResultVo resultVo = new ResultVo();

        userService.modifyRole(userRoleVo);
        resultVo.setCode(0);
        resultVo.setMsg("修改成功");

        return resultVo;
    }

    @PutMapping("/userInfo")
    public ResultVo changeUserInfo(@RequestBody UserInfoVo userInfoVo) {
        ResultVo resultVo = new ResultVo();

        userService.modifyUserInfo(userInfoVo);

        resultVo.setCode(ResultVo.SUCCESS);
        resultVo.setMsg("修改成功");

        return resultVo;
    }

    @PutMapping("/pwd")
    public ResultVo resetPwd(@RequestBody ResetPwdVo resetPwdVo) {
        ResultVo resultVo = new ResultVo();

        userService.resetPwd(resetPwdVo.getPhoneNumber());

        resultVo.setCode(ResultVo.SUCCESS);
        resultVo.setMsg("重置成功");

        return resultVo;
    }

    @PostMapping("/user")
    public ResultVo createUser(@RequestBody CreateUserVo createUserVo) {
        ResultVo resultVo = new ResultVo();

        userService.addUser(createUserVo);

        resultVo.setCode(ResultVo.SUCCESS);
        resultVo.setMsg("创建成功");

        return resultVo;
    }


//    @PostMapping("/signup")
//    public ResultVo signup(@RequestBody SignupVo signupVo) {
//        ResultVo resultVo = new ResultVo();
//
//        resultVo.setCode(ResultVo.SUCCESS);
//        boolean result = userService.addUser(signupVo);
//        if (result) {
//            resultVo.setCode(ResultVo.SUCCESS);
//            resultVo.setMsg("注册成功");
//        } else {
//            resultVo.setCode(ResultVo.FAILED);
//            resultVo.setMsg("注册失败");
//        }
//        return resultVo;
//    }

}
