package com.example.survey.dao.impl;

import com.example.survey.dao.InvestigationRecordDao;
import com.example.survey.entity.InvestigationRecord;
import com.example.survey.entity.inner.OperationInformation;
import com.example.survey.exception.RecordException;
import lombok.extern.log4j.Log4j2;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.ArrayOperators;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * @author Pope
 */
@Log4j2
@Repository
public class InvestigationRecordDaoImpl implements InvestigationRecordDao {

    @Autowired
    MongoTemplate mongoTemplate;

    @Override
    public boolean insertInvestigationRecord(InvestigationRecord record) {
        try {
            mongoTemplate.save(record);
        } catch (Exception e) {
            log.error("调查记录插入失败");
            log.error(e.getMessage());
            throw new RecordException("已存在对应调查对象的调查记录");
        }
        return true;
    }

    @Override
    public List<InvestigationRecord> listInvestigationRecordLimit(String investigatorPhone, int offset, int number, String state, String idNumber, String version, String questionnaireNumber, Boolean diseased) {
        ;

        Criteria criteria = new Criteria();
        //流调人员电话号码
        if (investigatorPhone != null) {
            criteria.and("investigatorPhone").is(investigatorPhone);
        }

        //调查记录状态
        if (state != null) {
            criteria.and("state").is(state);
        } else {
            criteria.and("state").in(InvestigationRecord.UNDER_REVIEW, InvestigationRecord.REVIEWED);
        }

        //调查对象身份证号
        if (idNumber != null) {
            criteria.and("idNumber").is(idNumber);
        }

        //版本
        if (version != null) {
            criteria.and("version").is(version);
        }

        //问卷编号
        if (questionnaireNumber != null) {
            criteria.and("questionnaireNumber").is(questionnaireNumber);
        }

        //是否患病
        if (diseased != null) {
            criteria.and("diseased").is(diseased);
        }
        Query query = new Query(criteria);
        query = query.skip(offset).limit(number);
        return mongoTemplate.find(query, InvestigationRecord.class);

    }

    @Override
    public long countInvestigationRecordByUserPhone(String userPhone, String state) {
        Query query = new Query(Criteria.where("userPhone").is(userPhone).and("state").is(state));
        return mongoTemplate.count(query, InvestigationRecord.class);
    }

    @Override
    public void updateRecordState(String idNumber, String oldState, String newState) {
        Query query = new Query(Criteria.where("idNumber").is(idNumber).and("state").is(oldState));
        Update update = new Update().set("state", newState);
        mongoTemplate.updateFirst(query, update, InvestigationRecord.class);
    }

    @Override
    public InvestigationRecord selectInvestigationRecord(String idNumber, String state) {
        Query query = new Query(Criteria.where("idNumber").is(idNumber).and("state").is(state));
        return mongoTemplate.findOne(query, InvestigationRecord.class);
    }

    @Override
    public void reviewRecord(InvestigationRecord record) {
        Query query = new Query(Criteria.where("idNumber").is(record.getIdNumber()).and("state").is(InvestigationRecord.UNDER_REVIEW));
        Update update = new Update()
                .set("state", record.getState())
                .set("operationInformationList", record.getOperationInformationList());
        mongoTemplate.updateFirst(query, update, InvestigationRecord.class);
    }

    @Override
    public boolean existInvestigationRecord(String idNumber, String... states) {
        Query query = new Query(Criteria.where("idNumber").is(idNumber).and("state").in(states));
        return mongoTemplate.exists(query, InvestigationRecord.class);
    }

    @Override
    public void updateManyRecordUserPhone(String oldUserPhone, String newUserPhone) {
        Query query = new Query(Criteria.where("userPhone").is(oldUserPhone));
        Update update = new Update().set("userPhone", newUserPhone);
        mongoTemplate.updateMulti(query, update, InvestigationRecord.class);
    }


}
