package com.example.survey.dao.impl;

import com.example.survey.dao.UserDao;
import com.example.survey.entity.Role;
import com.example.survey.entity.User;
import com.example.survey.exception.UserException;
import com.example.survey.vo.UserInfoVo;
import com.mongodb.client.result.DeleteResult;
import lombok.extern.log4j.Log4j2;
import org.bson.types.ObjectId;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.data.mongodb.core.query.Update;
import org.springframework.stereotype.Repository;
import org.springframework.stereotype.Service;

import java.util.Iterator;
import java.util.List;


/**
 * @author Pope
 */
@Log4j2
@Repository
public class UserDaoImpl implements UserDao {
    @Autowired
    private MongoTemplate mongoTemplate;

    @Override
    public User selectUser(String phoneNumber, String password) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber).and("password").is(password));
        return mongoTemplate.findOne(query, User.class);
    }

    @Override
    public boolean insertUser(User user) {
        try {
            mongoTemplate.save(user);
        } catch (Exception e) {
            log.error("用户插入失败");
            log.error(e.getMessage());
            throw new UserException("已存在用户");
        }
        return true;
    }

    @Override
    public List<User> listUserLimit(String username, String phoneNumber, int offset, int limit) {
        Criteria criteria = new Criteria();

        //用户名模糊查询
        if (username != null) {
            criteria.and("username").regex(username);
        }

        //电话号码精确查询
        if (phoneNumber != null) {
            criteria.and("phoneNumber").is(phoneNumber);
        }

        Query query = new Query(criteria).skip(offset).limit(limit);
        return mongoTemplate.find(query, User.class);
    }

    @Override
    public boolean existUser(String phoneNumber) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber));

        return mongoTemplate.exists(query, User.class);
    }

    @Override
    public void deleteUser(String phoneNumber) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber));
        mongoTemplate.remove(query, User.class);
    }

    @Override
    public void clearRole(Role role) {
        Query query = new Query(Criteria.where("roleList").elemMatch(Criteria.where("$id").is(role.getId())));
        List<User> users = mongoTemplate.find(query, User.class);
        for (User user : users) {
            List<Role> roleList = user.getRoleList();
            Iterator<Role> it = roleList.iterator();
            while (it.hasNext()) {
                if (role.getName().equals(it.next().getName())) {
                    it.remove();
                    break;
                }
            }
            Query userQuery = new Query(Criteria.where("phoneNumber").is(user.getPhoneNumber()));
            Update update = new Update().set("roleList", roleList);
            mongoTemplate.updateFirst(userQuery, update, User.class);
        }
    }

    @Override
    public void updateUserRole(String phoneNumber, List<Role> roleList) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber));
        User user = mongoTemplate.findOne(query, User.class);
        if (user == null) {
            throw new UserException("用户不存在");
        }
        user.setRoleList(roleList);
        mongoTemplate.save(user);
    }

    @Override
    public String selectUsername(String phoneNumber) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber));
        User user = mongoTemplate.findOne(query, User.class);
        if(user == null){
            log.error("用户不存在");
            throw new UserException("用户不存在");
        }
        return user.getUsername();
    }

    @Override
    public User selectUser(String phoneNumber) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber));
        return mongoTemplate.findOne(query, User.class);
    }

    @Override
    public void resetPwd(String phoneNumber) {
        Query query = new Query(Criteria.where("phoneNumber").is(phoneNumber));
        Update update = new Update().set("password", "123456");

        mongoTemplate.updateFirst(query, update, User.class);

    }

    @Override
    public void modifyUser(UserInfoVo userInfoVo) {
        Query query = new Query(Criteria.where("phoneNumber").is(userInfoVo.getPhoneNumber()));
        Update update = new Update()
                .set("username", userInfoVo.getUsername())
                .set("idNumber", userInfoVo.getIdNumber())
                .set("administrativeArea", userInfoVo.getAdministrativeArea());
        mongoTemplate.updateFirst(query, update, User.class);
    }


}
