package com.example.survey.entity;

import com.example.survey.entity.inner.AdministrativeArea;
import com.example.survey.vo.CreateUserVo;
import lombok.*;
import org.bson.types.ObjectId;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.index.Indexed;
import org.springframework.data.mongodb.core.mapping.DBRef;
import org.springframework.data.mongodb.core.mapping.Document;

import java.util.*;

/**
 * @author Pope
 */
@Getter
@Setter
@ToString
@NoArgsConstructor
@AllArgsConstructor
@Document(collection = "user")
public class User {

    /**
     * id
     */
    @Id
    private ObjectId id;

    /**
     * 用户名
     */
    private String username;

    /**
     * 密码
     */
    private String password;

    /**
     * 身份证号
     */
    private String idNumber;

    /**
     * 行政区划
     */
    private AdministrativeArea administrativeArea;

    /**
     * 手机号
     */
    @Indexed(unique = true)
    private String phoneNumber;

    /**
     * 一个人可以拥有多个角色
     */
    @DBRef
    private List<Role> roleList;

    /**
     * 一个人所属的部门
     */
    @DBRef
    private List<Department> departmentList;


    public User(CreateUserVo vo) {
        username = vo.getUsername();
        password = vo.getPassword();
        idNumber = vo.getIdNumber();
        phoneNumber = vo.getPhoneNumber();
        roleList = new LinkedList<>();
        departmentList = new LinkedList<>();
    }

}
