package com.example.survey.service;

import com.example.survey.entity.InvestigationRecord;
import com.example.survey.vo.InvestigationRecordVo;
import com.example.survey.vo.ReviewVo;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author Pope
 */
public interface InvestigationRecordService {

    /**
     * 创建新的调查记录
     *
     * @param vo 调查记录信息
     * @return 是否创建成功
     */
    boolean addInvestigationRecord(InvestigationRecordVo vo);

    /**
     * 根据筛选条件查询调查记录
     *
     * @param userPhone           用户电话号码
     * @param currentPage         当前页数
     * @param pageSize            页大小
     * @param state               状态
     * @param idNumber            调查对象身份证号
     * @param version             版本
     * @param questionnaireNumber 问卷编号
     * @param diseased            是否患病
     * @return 调查记录列表
     */
    List<InvestigationRecord> listUnderReviewRecordLimit(String userPhone, int currentPage, int pageSize, String state, String idNumber, String version, String questionnaireNumber, Boolean diseased);

    /**
     * 根据流调人员电话号码查询待审核记录数量
     *
     * @param userPhone 用户电话号码
     * @return 记录数量
     */
    long countUnderReviewRecord(String userPhone);

    /**
     * 提交修改后的调查记录，等待审核
     *
     * @param vo 调查记录Vo
     * @return 修改是否提交成功
     */
    void changeInvestigationRecord(InvestigationRecordVo vo);

    /**
     * 审核待调查记录
     *
     * @param reviewVo 审核信息
     * @return 是否成功
     */
    boolean reviewRecord(ReviewVo reviewVo);

    /**
     * 导出为word
     *
     * @param idNumber 调查对象身份证号
     * @param response 响应
     */
    void export2Word(String idNumber, HttpServletResponse response);
}
