package com.example.survey.service;

import com.example.survey.dto.LoginDto;
import com.example.survey.dto.UserDto;
import com.example.survey.vo.LoginVo;
import com.example.survey.vo.CreateUserVo;
import com.example.survey.vo.UserInfoVo;
import com.example.survey.vo.UserRoleVo;

import java.util.List;

/**
 * @author Pope
 */
public interface UserService {
    /**
     * 根据登录信息判断是否登录
     *
     * @param loginVo 登录信息
     * @return 用户信息  若登录失败则返回null
     */
    LoginDto matchAuth(LoginVo loginVo);

    /**
     * 注册用户
     *
     * @param createUserVo 注册信息
     * @return 是否注册成功
     */
    boolean addUser(CreateUserVo createUserVo);

    /**
     * 修改用户权限
     *
     * @param userRoleVo 新权限信息
     */
    void modifyRole(UserRoleVo userRoleVo);

    /**
     * 根据筛选条件分页查询用户
     *
     * @param username    用户名  模糊匹配
     * @param phoneNumber 电话号码  精确匹配
     * @param currentPage 当前页数
     * @param pageSize    页大小
     * @return 用户信息dto
     */
    List<UserDto> listUserLimit(String username, String phoneNumber, int currentPage, int pageSize);

    /**
     * 删除用户
     *
     * @param phoneNumber 电话号码
     * @return 是否删除成功
     */
    boolean deleteUser(String phoneNumber);

    /**
     * 修改用户信息
     *
     * @param userInfoVo 用户信息
     */
    void modifyUserInfo(UserInfoVo userInfoVo);

    /**
     * 重置密码
     *
     * @param phoneNumber 电话号码
     */
    void resetPwd(String phoneNumber);
}
