package com.example.survey.service.impl;

import com.example.survey.dao.RoleDao;
import com.example.survey.dao.UserDao;
import com.example.survey.entity.Role;
import com.example.survey.enumeration.AuthEnum;
import com.example.survey.exception.AuthException;
import com.example.survey.service.RoleService;
import com.example.survey.util.TokenUtil;
import com.example.survey.vo.CreateRoleVo;
import com.example.survey.vo.ModifyRoleVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @author Pope
 */
@Service
public class RoleServiceImpl implements RoleService {

    @Autowired
    private RoleDao roleDao;

    @Autowired
    private UserDao userDao;

    @Override
    public boolean addRole(CreateRoleVo createRoleVo) {
        Role role = new Role();
        role.setName(createRoleVo.getRoleName());
        Set<AuthEnum> authoritySet = new HashSet<>();
        for (String name : createRoleVo.getAuthList()) {
            authoritySet.add(AuthEnum.getRoleEnum(name));
        }
        role.setAuthoritySet(authoritySet);

        return roleDao.insertRole(role);
    }

    @Override
    public List<CreateRoleVo> getRole(String roleName, int currentPage, int pageSize) {
        List<Role> roleList = roleDao.listLimitRole(roleName, currentPage * pageSize, pageSize);
        List<CreateRoleVo> roleVoList = new LinkedList<>();
        for (Role role : roleList) {
            roleVoList.add(new CreateRoleVo(role));
        }

        return roleVoList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteRole(String roleName) {
        //将用户数据库中含有该角色的用户更新
        userDao.clearRole(roleDao.selectRole(roleName));
        //从角色数据库删除角色
        roleDao.deleteRole(roleName);

        //删除redis中的角色
        TokenUtil.expireKey(roleName);
    }

    @Override
    public void modifyRole(ModifyRoleVo modifyRoleVo) {
        Set<AuthEnum> authEnumList = new HashSet<>();
        Set<String> routeSet = new HashSet<>();
        for (String authName : modifyRoleVo.getAuthList()) {
            AuthEnum authEnum = AuthEnum.getRoleEnum(authName);
            if(authEnum == null){
                throw new AuthException("权限不存在");
            }
            authEnumList.add(authEnum);
            routeSet.addAll(authEnum.getRoutes());
        }
        roleDao.updateRole(modifyRoleVo.getRoleName(), authEnumList);
        TokenUtil.setWithoutExpireTime(modifyRoleVo.getRoleName(), routeSet);
    }
}
