package com.example.survey.util;

import com.example.survey.dao.UserDao;
import com.example.survey.entity.InvestigationRecord;
import com.example.survey.entity.inner.*;
import org.apache.poi.xwpf.usermodel.*;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author Pope
 */
@Component
public class WordUtil {

    @Autowired
    private UserDao userDao;

    private static WordUtil wordUtil;

    @PostConstruct
    public void init() {
        wordUtil = this;
        wordUtil.userDao = this.userDao;
    }

    /**
     * 分隔符
     */
    private static final String SEPARATOR = "        ";

    private static final String OTHER = "other";

    public static void writeDocx(InvestigationRecord record, HttpServletResponse response) throws Exception {
        XWPFDocument document = new XWPFDocument();
        int index = 1;

        //添加标题
        XWPFParagraph title = document.createParagraph();
        title.setAlignment(ParagraphAlignment.CENTER);
        XWPFRun titleRun = title.createRun();
        titleRun.setText("新型冠状病毒肺炎病例个案调查表");
        //设置字体
        setRunFont(titleRun, "宋体", 18, true);

        //换行
        document.createParagraph().createRun().setText("\r");

        //问卷编号与身份证号
        XWPFParagraph firstPara = document.createParagraph();
        firstPara.setAlignment(ParagraphAlignment.CENTER);
        XWPFRun firstParaRun = firstPara.createRun();
        firstParaRun.setText("问卷编号：" + record.getQuestionnaireNumber() + SEPARATOR + "身份证号：" + record.getIdNumber());
        //设置字体
        setRunFont(firstParaRun, "仿宋", 10, false);

        //换行
        document.createParagraph().createRun().setText("\r");

        //信息表格
        XWPFTable infoTable = document.createTable();
        //列宽自动分割
        CTTblWidth infoTableWidth = infoTable.getCTTbl().addNewTblPr().addNewTblW();
        infoTableWidth.setType(STTblWidth.DXA);
        infoTableWidth.setW(BigInteger.valueOf(9072));


        //表格第一行  基本信息
        XWPFTableRow infoTableRow1 = infoTable.getRow(0);
        XWPFTableCell basicInfoCell = infoTableRow1.getCell(0);
        basicInfoCell.removeParagraph(0);

        //基本信息标题
        createParaInCell(basicInfoCell, "一、基本信息", true);
        createBreakLine(basicInfoCell);

        //姓名    性别
        createParaInCell(basicInfoCell, index++ + ".姓名：" + record.getName() + SEPARATOR + index++ + ".性别：" + record.getGender(), false);
        //是否为境外输入病例
        createParaInCell(basicInfoCell, index++ + ".是否为境外输入病例：" + (record.isOverseasCase() ? "是" : "否"), false);
        if (record.isOverseasCase()) {
            //如果为境外输入病例，填写入境信息
            EntryExperience entryExperience = record.getEntryExperience();
            createParaInCell(basicInfoCell, SEPARATOR + "入境前居住或旅行的国家或地区：" + entryExperience.getAreaOfResidence(), false);
            createParaInCell(basicInfoCell, SEPARATOR + "入境前途经国家和地区：" + entryExperience.getAreaOfAccess(), false);
            createParaInCell(basicInfoCell, SEPARATOR + "国籍：" + entryExperience.getNationality() + SEPARATOR + "护照号码：" + entryExperience.getPassportNumber(), false);
            createParaInCell(basicInfoCell, SEPARATOR + "入境口岸：" + entryExperience.getEntryPort().getProvince() + "省" + entryExperience.getEntryPort().getPort(), false);

            createParaInCell(basicInfoCell, SEPARATOR + "入境日期：" + formatDate(entryExperience.getEntryDate()), false);
            createParaInCell(basicInfoCell, SEPARATOR + "入境交通方式：" + entryExperience.getTransportation(), false);
        }

        //病例发现与就诊
        XWPFTableRow infoTableRow2 = infoTable.createRow();
        XWPFTableCell hospInfoCell = infoTableRow2.getCell(0);
        hospInfoCell.removeParagraph(0);

        //病例发现与就诊标题
        createParaInCell(hospInfoCell, "二、病例发现与就诊", true);
        createBreakLine(hospInfoCell);

        //病例发现途径
        createParaInCell(hospInfoCell, index++ + ".病例发现途径：" + record.getWayOfBeingDiscovered(), false);
        //入院日期
        createParaInCell(hospInfoCell, index++ + ".入院日期：" + formatDate(record.getDateOfAdmission()), false);
        StringBuilder symptomAndSign = new StringBuilder();
        //入院时症状和体征
        for (String s : record.getSymptomAndSign()) {
            symptomAndSign.append(s).append("  ");
        }
        createParaInCell(hospInfoCell, index++ + ".入院症状和体征：" + symptomAndSign.toString(), false);
        //有无并发症
        createParaInCell(hospInfoCell, index++ + ".有无并发症：" + (record.isExistComplication() ? "有" : "无"), false);
        if (record.getComplication() != null) {
            StringBuilder complication = new StringBuilder("并发症：");
            for (String s : record.getComplication()) {
                complication.append(s);
            }
            createParaInCell(hospInfoCell, complication.toString(), false);
            if (record.getComplication().contains("发热")) {
                //如果并发症包含发热，则打印最高温度
                createParaInCell(hospInfoCell, SEPARATOR + "发热：最高温度" + record.getMaxTemp() + "°C", false);
            }

        }
        //胸部Ｘ线或CT检测是否有肺炎影像学特征
        createParaInCell(hospInfoCell, index++ + ".胸部Ｘ线或CT检测是否有肺炎影像学特征：" + (record.isExistImagingFeatureOfPneumonia() ? "是" : "否"), false);
        if (record.isExistImagingFeatureOfPneumonia()) {
            //如果存在肺炎影像学特征，打印检测时间
            createParaInCell(hospInfoCell, SEPARATOR + "检测时间：" + formatDate(record.getTestDate()), false);
        }
        //出院日期
        createParaInCell(hospInfoCell, index++ + ".出院日期：" + formatDate(record.getDateOfDischarge()), false);
        //是否为以下特定职业人群
        createParaInCell(hospInfoCell, index++ + ".是否为以下特定职业人群：" + record.getSpecificOccupation(), false);
        if (OTHER.equals(record.getSpecificOccupation())) {
            //若职业为其他
            createParaInCell(hospInfoCell, SEPARATOR + "职业：" + record.getOtherSpecificOccupation(), false);
        }
        //如为医务人员，请选择具体工作性质：
        createParaInCell(hospInfoCell, SEPARATOR + "如为医务人员，请选择具体工作性质：" + record.getWorkInformation(), false);
        if (OTHER.equals(record.getWorkInformation())) {
            //若工作性质为其他
            createParaInCell(hospInfoCell, SEPARATOR + "其他工作性质：" + record.getOtherWorkInformation(), false);
        }
        //是否为孕妇
        createParaInCell(hospInfoCell, index++ + ".是否为孕妇：" + (record.isPregnant() ? "是" : "否"), false);
        if (record.isPregnant()) {
            //如果是孕妇，打印孕周
            createParaInCell(hospInfoCell, SEPARATOR + "孕周：" + record.getPregnantWeek(), false);
        }
        //既往病史和基本情况（可多选）
        StringBuilder medicalHistory = new StringBuilder();
        for (String s : record.getMedicalHistory()) {
            medicalHistory.append(s).append("  ");
        }
        createParaInCell(hospInfoCell, index++ + ".既往病史与基本情况：" + medicalHistory, false);
        //既往病史描述
        createParaInCell(hospInfoCell, SEPARATOR + "既往病史描述：" + record.getMedicalHistoryDetail(), false);


        //发病或检测阳性前 14 天内是否有以下暴露史或接触史：
        XWPFTableRow infoTableRow3 = infoTable.createRow();
        XWPFTableCell contactHistoryCell = infoTableRow3.getCell(0);
        contactHistoryCell.removeParagraph(0);

        //病例发现与就诊标题
        createParaInCell(contactHistoryCell, "三、发病或检测阳性前 14 天内是否有以下暴露史或接触史：", true);
        createBreakLine(contactHistoryCell);

        //是否有境外疫情严重国家或地区的旅行史或居住史
        createParaInCell(contactHistoryCell, index++ + ".是否有境外疫情严重国家或地区的旅行史或居住史：" + (record.isNearSeriousRegion() ? "是" : "否"), false);

        //是否接触过来自境外疫情严重的国家或地区的发热或有呼吸道症状的患者
        createParaInCell(contactHistoryCell, index++ + ".是否接触过来自境外疫情严重的国家或地区的发热或有呼吸道症状的患者：" + (record.isContactWithSeriousRegionPatients() ? "是" : "否"), false);

        //是否曾有确诊病例或无症状感染者的接触史
        createParaInCell(contactHistoryCell, index++ + ".是否曾有确诊病例或无症状感染者的接触史：" + (record.isContactWithConfirmedCase() ? "是" : "否"), false);

        //患者同一家庭、办公室、学校或托幼机构班级、车间等集体单位是否有聚集性发病
        createParaInCell(contactHistoryCell, index++ + ".患者同一家庭、办公室、学校或托幼机构班级、车间等集体单位是否有聚集性发病：" + (record.isAggregationDisease() ? "是" : "否"), false);

        //体温记录
        XWPFTableRow infoTableRow4 = infoTable.createRow();
        XWPFTableCell tempHistory = infoTableRow4.getCell(0);
        tempHistory.removeParagraph(0);
        createParaInCell(tempHistory, "四、体温记录", true);
        createBreakLine(tempHistory);

        //体温记录表
        XWPFTable tempTable = createTableInCell(tempHistory);

        //创建表头
        XWPFTableRow tempTableHead = tempTable.createRow();
        XWPFTableCell tempDateHead = tempTableHead.createCell();
        tempDateHead.removeParagraph(0);
        createParaInCell(tempDateHead, "记录时间", false);
        setCellCenter(tempDateHead);

        XWPFTableCell tempHead = tempTableHead.createCell();
        tempHead.removeParagraph(0);
        createParaInCell(tempHead, "温度", false);
        setCellCenter(tempHead);
        //创建数据
        for (TempHistory history : record.getTempHistoryList()) {
            XWPFTableRow row = tempTable.createRow();

            XWPFTableCell temp = row.getCell(0);
            temp.removeParagraph(0);
            createParaInCell(temp, history.getData(), false);
            setCellCenter(temp);

            XWPFTableCell date = row.getCell(1);
            date.removeParagraph(0);
            createParaInCell(date, formatTime(history.getTime()), false);
            setCellCenter(date);
        }


        //居住史
        XWPFTableRow infoTableRow5 = infoTable.createRow();
        XWPFTableCell liveAndTravelHistory = infoTableRow5.getCell(0);
        liveAndTravelHistory.removeParagraph(0);

        createParaInCell(liveAndTravelHistory, "五、居旅史", true);
        createBreakLine(liveAndTravelHistory);

        List<LiveHistory> liveHistoryList = record.getLiveHistoryList();
        if (liveHistoryList != null) {
            for (int i = 0; i < liveHistoryList.size(); i++) {
                createParaInCell(liveAndTravelHistory, "居住史" + (i + 1), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "开始时间：" + formatDate(liveHistoryList.get(i).getBeginTime()), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "地点：" + liveHistoryList.get(i).getLocation().getMsg(), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "结束时间：" + formatDate(liveHistoryList.get(i).getEndTime()), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "详细信息：" + liveHistoryList.get(i).getDetail(), false);
            }
        }
        List<TravelHistory> travelHistoryList = record.getTravelHistoryList();
        if (travelHistoryList != null) {
            for (int i = 0; i < travelHistoryList.size(); i++) {
                createParaInCell(liveAndTravelHistory, "旅行史" + (i + 1), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "开始时间：" + formatDate(travelHistoryList.get(i).getBeginTime()), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "开始地点：" + travelHistoryList.get(i).getStartLocation().getMsg(), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "结束时间：" + formatDate(travelHistoryList.get(i).getEndTime()), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "结束地点：" + travelHistoryList.get(i).getEndLocation().getMsg(), false);
                createParaInCell(liveAndTravelHistory, SEPARATOR + "详细信息：" + travelHistoryList.get(i).getDetail(), false);
            }
        }


        //实验室检测单
        XWPFTableRow infoTableRow6 = infoTable.createRow();
        XWPFTableCell detectionList = infoTableRow6.getCell(0);
        detectionList.removeParagraph(0);
        createParaInCell(detectionList, "六、实验室检测单", true);
        createBreakLine(detectionList);

        //实验室检测单表格
        XWPFTable detectionTable = createTableInCell(detectionList);

        //创建表头
        XWPFTableRow detectionTableHead = detectionTable.createRow();

        XWPFTableCell nameHead = detectionTableHead.createCell();
        nameHead.removeParagraph(0);
        createParaInCell(nameHead, "标本类型：", false);
        setCellCenter(nameHead);

        XWPFTableCell detectionDate = detectionTableHead.createCell();
        detectionDate.removeParagraph(0);
        createParaInCell(detectionDate, "采样时间：", false);
        setCellCenter(detectionDate);

        XWPFTableCell detectionResult = detectionTableHead.createCell();
        detectionResult.removeParagraph(0);
        createParaInCell(detectionResult, "采样结果：", false);
        setCellCenter(detectionResult);

        for (Detection detection : record.getDetectionList()) {
            XWPFTableRow row = detectionTable.createRow();

            XWPFTableCell name = row.getCell(0);
            name.removeParagraph(0);
            createParaInCell(name, detection.getName(), false);
            setCellCenter(name);

            XWPFTableCell date = row.getCell(1);
            date.removeParagraph(0);
            createParaInCell(date, formatDate(detection.getDate()), false);
            setCellCenter(date);

            XWPFTableCell result = row.getCell(2);
            result.removeParagraph(0);
            createParaInCell(result, detection.getResult(), false);
            setCellCenter(result);
        }


        //调查信息
        XWPFParagraph surveyInfo = document.createParagraph();
        XWPFRun surveyInfoRun = surveyInfo.createRun();
        String surveyInfoStr = "调查单位：" + record.getInvestigationCompany() + "  " +
                "调查者：" + wordUtil.userDao.selectUsername(record.getUserPhone()) + "  " +
                "调查时间：" + formatDate(record.getInvestigationDate());
        surveyInfoRun.setText(surveyInfoStr);
        setRunFont(surveyInfoRun, "仿宋", 11, false);


        //八进制输出流
        response.setContentType("application/octet-stream");

        //这后面可以设置导出Excel的名称，此例中名为student.xls
        response.setHeader("Content-disposition", "attachment;filename=" + record.getIdNumber() + ".docx");
        document.write(response.getOutputStream());
    }


    public static XWPFTable createTableInCell(XWPFTableCell cell) {
        cell.addParagraph();
        XWPFTable table = cell.insertNewTbl(cell.getParagraphArray(1).getCTP().newCursor());
        table.getCTTbl().addNewTblPr().addNewTblBorders()
                .addNewLeft().setVal(STBorder.SINGLE);
        table.getCTTbl().addNewTblPr().addNewTblBorders()
                .addNewRight().setVal(STBorder.SINGLE);
        table.getCTTbl().addNewTblPr().addNewTblBorders()
                .addNewTop().setVal(STBorder.SINGLE);
        table.getCTTbl().addNewTblPr().addNewTblBorders()
                .addNewBottom().setVal(STBorder.SINGLE);
        table.getCTTbl().addNewTblPr().addNewTblBorders()
                .addNewInsideH().setVal(STBorder.SINGLE);
        table.getCTTbl().addNewTblPr().addNewTblBorders()
                .addNewInsideV().setVal(STBorder.SINGLE);

        //设置表格水平居中
        table.getCTTbl().getTblPr().addNewJc().setVal(STJc.CENTER);
        //设置表格垂直居中
        cell.setVerticalAlignment(XWPFTableCell.XWPFVertAlign.CENTER);
        table.setWidth(8000);

        return table;
    }

    /**
     * 设置单元格内容居中
     */
    public static void setCellCenter(XWPFTableCell cell) {
        CTTc cttc = cell.getCTTc();
        CTTcPr ctPr = cttc.addNewTcPr();
        ctPr.addNewVAlign().setVal(STVerticalJc.CENTER);
        cttc.getPList().get(0).addNewPPr().addNewJc().setVal(STJc.CENTER);
    }

    /**
     * 在单元格中创建段落
     */
    public static void createParaInCell(XWPFTableCell cell, String text, boolean title) {
        XWPFParagraph paragraph = cell.addParagraph();
        paragraph.setAlignment(ParagraphAlignment.LEFT);
        XWPFRun run = paragraph.createRun();
        run.setText(text);
        if (title) {
            setRunFont(run, "仿宋", 14, true);
        } else {
            setRunFont(run, "仿宋", 11, false);
        }
    }

    /**
     * 换行
     */
    public static void createBreakLine(XWPFTableCell cell) {
        cell.addParagraph().createRun().setText("\r");
    }

    /**
     * 设置段落字体
     */
    public static void setRunFont(XWPFRun run, String font, int fontSize, boolean bold) {
        run.setFontFamily(font);
        run.getCTR().addNewRPr().addNewRFonts().setEastAsia(font);
        run.setBold(bold);
        run.setFontSize(fontSize);
    }

    /**
     * 根据日期提取时间
     *
     * @param date 日期
     * @return 格式化后的时间字符串 年-月-日
     */
    public static String formatDate(Date date) {
        StringBuilder res = new StringBuilder();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        res.append(calendar.get(Calendar.YEAR)).append("年");
        res.append(calendar.get(Calendar.MONTH) + 1).append("月");
        res.append(calendar.get(Calendar.DAY_OF_MONTH)).append("日");

        return res.toString();
    }

    /**
     * 根据日期提取时间
     *
     * @param date 日期
     * @return 格式化后的时间字符串 年-月-日-时-分-秒
     */
    public static String formatTime(Date date) {
        StringBuilder res = new StringBuilder();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);
        res.append(calendar.get(Calendar.YEAR)).append("年");
        res.append(calendar.get(Calendar.MONTH) + 1).append("月");
        res.append(calendar.get(Calendar.DAY_OF_MONTH)).append("日");
        res.append(calendar.get(Calendar.HOUR_OF_DAY)).append("时");
        res.append(calendar.get(Calendar.MINUTE)).append("分");
        res.append(calendar.get(Calendar.SECOND)).append("秒");

        return res.toString();
    }

}
