package com.example.survey.vo;

import com.example.survey.entity.inner.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.*;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Pope
 */
@Getter
@Setter
@ToString
@NoArgsConstructor
@AllArgsConstructor
public class InvestigationRecordVo {
    /**
     * 问卷编号
     */
    private String questionnaireNumber;

    /**
     * 身份证号
     * 唯一索引
     */
    private String idNumber;

    //==================================基本信息==================================

    /**
     * 姓名
     */
    private String name;

    /**
     * 性别
     * true为男性，false为女性
     */
    private String gender;

    /**
     * 是否为境外病例
     */
    private boolean overseasCase;

    /**
     * 是否为病例
     * 区分病例与密切接触者
     */
    private boolean diseased;

    /**
     * 入境经历
     * 选填
     */
    private EntryExperience entryExperience;

    //==================================病例发现与就诊==================================

    /**
     * 病例被发现途径
     */
    private String wayOfBeingDiscovered;

    /**
     * 入院时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date dateOfAdmission;

    /**
     * 入院时症状和体征
     */
    private List<String> symptomAndSign;

    /**
     * 有无并发症
     */
    private boolean existComplication;

    /**
     * 并发症
     * 选填
     */
    private List<String> complication;

    /**
     * 当并发症存在发热时，记录最高体温
     */
    private String maxTemp;

    /**
     * 胸部Ｘ线或 CT 检测是否有肺炎影像学特征
     */
    private boolean existImagingFeatureOfPneumonia;

    /**
     * 如果肺炎影像学特征为是，此处填检测时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date testDate;

    /**
     * 出院日期
     * 年月日
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date dateOfDischarge;

    //==================================危险因素与暴露史==================================

    /**
     * 是否为特定职业人群
     */
    private String specificOccupation;

    /**
     * 当specificOccupation为other时起作用
     */
    private String otherSpecificOccupation;

    /**
     * 如为医护人员，填写具体工作性质
     */
    private String workInformation;

    /**
     * 当workInformation为other时起作用
     */
    private String otherWorkInformation;

    /**
     * 是否为孕妇
     */
    private boolean pregnant;

    /**
     * 孕周
     */
    private int pregnantWeek;

    /**
     * 病史
     */
    private List<String> medicalHistory;

    /**
     * 对既往病史的详细描述
     */
    private String medicalHistoryDetail;

    //==================================发病或检测阳性前14天内是否有以下暴露史或接触史==================================

    /**
     * 是否有境外疫情严重国家或地区的旅行史或居住史
     */
    private boolean nearSeriousRegion;

    /**
     * 是否接触过来自境外疫情严重的国家或地区的发热或有呼吸道症状的患者
     */
    private boolean contactWithSeriousRegionPatients;

    /**
     * 是否曾有确诊病例或无症状感染者的接触史
     */
    private boolean contactWithConfirmedCase;

    /**
     * 患者同一家庭、办公室、学校或托幼机构班级、车间等集体单位是否有聚集性发病
     */
    private boolean aggregationDisease;


    //==================================实验室检测==================================

    /**
     * 实验室检测结果，未采集则value为null
     */
    private List<Detection> detectionList;

    /**
     * 调查单位
     */
    private String investigationCompany;

    /**
     * 调查者（身份证）
     * 外键
     */
    private String userPhone;

    /**
     * 调查日期
     * 年月日
     */
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date investigationDate;

    /**
     * 体温历史
     */
    private List<TempHistory> tempHistoryList;

    /**
     * 居住史
     */
    private List<LiveHistory> liveHistoryList;

    /**
     * 旅行史
     */
    private List<TravelHistory> travelHistoryList;

    /**
     * 备注
     */
    private String msg;

    /**
     * 存放一些url
     */
    private Map<String, String> urlMap;

}
